'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.PipelineGraph = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _StatusIndicator = require('../status/StatusIndicator');

var _SvgSpinner = require('../status/SvgSpinner');

var _TruncatingLabel = require('../TruncatingLabel');

var _PipelineGraphModel = require('./PipelineGraphModel');

var _PipelineGraphLayout = require('./PipelineGraphLayout');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// Generate a react key for a connection
// Fixme: Maybe refine this?

function connectorKey(leftNode, rightNode) {
    return 'c_' + leftNode.key + '_to_' + rightNode.key;
}

var PipelineGraph = exports.PipelineGraph = function (_Component) {
    _inherits(PipelineGraph, _Component);

    function PipelineGraph(props) {
        _classCallCheck(this, PipelineGraph);

        var _this = _possibleConstructorReturn(this, (PipelineGraph.__proto__ || Object.getPrototypeOf(PipelineGraph)).call(this, props));

        _this.state = {
            nodeColumns: [],
            connections: [],
            bigLabels: [],
            smallLabels: [],
            measuredWidth: 0,
            measuredHeight: 0,
            layout: _extends({}, _PipelineGraphModel.defaultLayout, props.layout),
            selectedStage: props.selectedStage
        };
        return _this;
    }
    // Flow typedefs


    _createClass(PipelineGraph, [{
        key: 'componentWillMount',
        value: function componentWillMount() {
            this.stagesUpdated(this.props.stages);
        }
    }, {
        key: 'componentWillReceiveProps',
        value: function componentWillReceiveProps(nextProps) {
            var _this2 = this;

            var newState = null; // null == no new state
            var needsLayout = false;

            if (nextProps.layout != this.props.layout) {
                newState = _extends({}, newState, { layout: _extends({}, _PipelineGraphModel.defaultLayout, this.props.layout) });
                needsLayout = true;
            }

            if (nextProps.selectedStage !== this.props.selectedStage) {
                // If we're just changing selectedStage, we don't need to re-generate the children
                newState = _extends({}, newState, { selectedStage: nextProps.selectedStage });
            }

            if (nextProps.stages !== this.props.stages) {
                needsLayout = true;
            }

            var doLayoutIfNeeded = function doLayoutIfNeeded() {
                if (needsLayout) {
                    _this2.stagesUpdated(nextProps.stages);
                }
            };

            if (newState) {
                // If we need to update the state, then we'll delay any layout changes
                this.setState(newState, doLayoutIfNeeded);
            } else {
                doLayoutIfNeeded();
            }
        }

        /**
         * Main process for laying out the graph. Calls out to PipelineGraphLayout module.
         */

    }, {
        key: 'stagesUpdated',
        value: function stagesUpdated() {
            var newStages = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

            this.setState((0, _PipelineGraphLayout.layoutGraph)(newStages, this.state.layout));
        }

        /**
         * Generate the Component for a big label
         */

    }, {
        key: 'renderBigLabel',
        value: function renderBigLabel(details) {
            var _state$layout = this.state.layout,
                nodeSpacingH = _state$layout.nodeSpacingH,
                labelOffsetV = _state$layout.labelOffsetV,
                connectorStrokeWidth = _state$layout.connectorStrokeWidth,
                ypStart = _state$layout.ypStart;


            var labelWidth = nodeSpacingH - connectorStrokeWidth * 2;
            var labelHeight = ypStart - labelOffsetV;
            var labelOffsetH = Math.floor(labelWidth / -2);

            // These are about layout more than appearance, so they should probably remain inline
            var bigLabelStyle = {
                position: 'absolute',
                width: labelWidth,
                maxHeight: labelHeight + 'px',
                textAlign: 'center',
                marginLeft: labelOffsetH
            };

            var x = details.x;
            var bottom = this.state.measuredHeight - details.y + labelOffsetV;

            // These are about layout more than appearance, so they're inline
            var style = _extends({}, bigLabelStyle, {
                bottom: bottom + 'px',
                left: x + 'px'
            });

            var classNames = ['pipeline-big-label'];
            if (this.stageIsSelected(details.stage) || this.stageChildIsSelected(details.stage)) {
                classNames.push('selected');
            }

            return _react2.default.createElement(
                _TruncatingLabel.TruncatingLabel,
                { className: classNames.join(' '), style: style, key: details.key },
                details.text
            );
        }

        /**
         * Generate the Component for a small label
         */

    }, {
        key: 'renderSmallLabel',
        value: function renderSmallLabel(details) {
            var _state$layout2 = this.state.layout,
                nodeSpacingH = _state$layout2.nodeSpacingH,
                nodeSpacingV = _state$layout2.nodeSpacingV,
                curveRadius = _state$layout2.curveRadius,
                connectorStrokeWidth = _state$layout2.connectorStrokeWidth,
                nodeRadius = _state$layout2.nodeRadius,
                smallLabelOffsetV = _state$layout2.smallLabelOffsetV;


            var smallLabelWidth = Math.floor(nodeSpacingH - 2 * curveRadius - 2 * connectorStrokeWidth); // Fit between lines
            var smallLabelHeight = Math.floor(nodeSpacingV - smallLabelOffsetV - nodeRadius - _SvgSpinner.strokeWidth);
            var smallLabelOffsetH = Math.floor(smallLabelWidth * -0.5);

            var x = details.x + smallLabelOffsetH;
            var top = details.y + smallLabelOffsetV;

            // These are about layout more than appearance, so they're inline
            var style = {
                top: top,
                left: x,
                position: 'absolute',
                width: smallLabelWidth,
                maxHeight: smallLabelHeight,
                textAlign: 'center'
            };

            var classNames = ['pipeline-small-label'];
            if (details.stage && this.stageIsSelected(details.stage)) {
                classNames.push('selected');
            }

            return _react2.default.createElement(
                _TruncatingLabel.TruncatingLabel,
                { className: classNames.join(' '), style: style, key: details.key },
                details.text
            );
        }

        /**
         * Generate SVG for a composite connection, which may be to/from many nodes.
         *
         * Farms work out to other methods on self depending on the complexity of the line required. Adds all the SVG
         * components to the elements list.
         */

    }, {
        key: 'renderCompositeConnection',
        value: function renderCompositeConnection(connection, elements) {
            var sourceNodes = connection.sourceNodes,
                destinationNodes = connection.destinationNodes,
                skippedNodes = connection.skippedNodes;


            if (skippedNodes.length === 0) {
                // Nothing too complicated, use the original connection drawing code
                this.renderBasicConnections(sourceNodes, destinationNodes, elements);
            } else {
                this.renderSkippingConnections(sourceNodes, destinationNodes, skippedNodes, elements);
            }
        }

        /**
         * Connections between adjacent columns without any skipping.
         *
         * Adds all the SVG components to the elements list.
         */

    }, {
        key: 'renderBasicConnections',
        value: function renderBasicConnections(sourceNodes, destinationNodes, elements) {
            var _state$layout3 = this.state.layout,
                connectorStrokeWidth = _state$layout3.connectorStrokeWidth,
                nodeSpacingH = _state$layout3.nodeSpacingH;

            var halfSpacingH = nodeSpacingH / 2;

            // Stroke props common to straight / curved connections
            var connectorStroke = {
                className: 'pipeline-connector',
                strokeWidth: connectorStrokeWidth
            };

            this.renderHorizontalConnection(sourceNodes[0], destinationNodes[0], connectorStroke, elements);

            if (sourceNodes.length === 1 && destinationNodes.length === 1) {
                return; // No curves needed.
            }

            // Work out the extents of source and dest space
            var rightmostSource = sourceNodes[0].x;
            var leftmostDestination = destinationNodes[0].x;

            for (var i = 1; i < sourceNodes.length; i++) {
                rightmostSource = Math.max(rightmostSource, sourceNodes[i].x);
            }

            for (var _i = 1; _i < destinationNodes.length; _i++) {
                leftmostDestination = Math.min(leftmostDestination, destinationNodes[_i].x);
            }

            // console.log(''); // TODO: RM
            // console.log('sourceNodes', sourceNodes.map(node => `${node.name} (${node.x})`).join(', ')); // TODO: RM
            // console.log('rightmostSource',rightmostSource); // TODO: RM
            // console.log('destNodes', destinationNodes.map(node => `${node.name} (${node.x})`).join(', ')); // TODO: RM
            // console.log('leftmostDestination',leftmostDestination); // TODO: RM

            // Collapse from previous node(s) to top column node
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = sourceNodes.slice(1)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var previousNode = _step.value;

                    var midPointX = Math.round((_PipelineGraphModel.MATRIOSKA_PATHS ? previousNode.x : rightmostSource) + halfSpacingH);
                    // console.log('collapse from',previousNode.name,'to',destinationNodes[0].name, 'mpx', midPointX); // TODO: RM
                    this.renderBasicCurvedConnection(previousNode, destinationNodes[0], midPointX, elements);
                }

                // Expand from top previous node to column node(s)
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = destinationNodes.slice(1)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var destNode = _step2.value;

                    var _midPointX = Math.round((_PipelineGraphModel.MATRIOSKA_PATHS ? destNode.x : leftmostDestination) - halfSpacingH);
                    // console.log('expand from',sourceNodes[0].name,'to',destNode.name, 'mpx', midPointX); // TODO: RM
                    this.renderBasicCurvedConnection(sourceNodes[0], destNode, _midPointX, elements);
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }
        }

        /**
         * Renders a more complex connection, that "skips" one or more nodes
         *
         * Adds all the SVG components to the elements list.
         */

    }, {
        key: 'renderSkippingConnections',
        value: function renderSkippingConnections(sourceNodes, destinationNodes, skippedNodes, elements) {
            var _state$layout4 = this.state.layout,
                connectorStrokeWidth = _state$layout4.connectorStrokeWidth,
                nodeRadius = _state$layout4.nodeRadius,
                terminalRadius = _state$layout4.terminalRadius,
                curveRadius = _state$layout4.curveRadius,
                nodeSpacingV = _state$layout4.nodeSpacingV,
                nodeSpacingH = _state$layout4.nodeSpacingH;


            var halfSpacingH = nodeSpacingH / 2;

            // Stroke props common to straight / curved connections
            var connectorStroke = {
                className: 'pipeline-connector',
                strokeWidth: connectorStrokeWidth
            };

            var skipConnectorStroke = {
                className: 'pipeline-connector-skipped',
                strokeWidth: connectorStrokeWidth
            };

            var lastSkippedNode = skippedNodes[skippedNodes.length - 1];
            var leftNode = void 0,
                rightNode = void 0;

            //--------------------------------------------------------------------------
            //  Draw the "ghost" connections to/from/between skipped nodes

            leftNode = sourceNodes[0];
            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
                for (var _iterator3 = skippedNodes[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    rightNode = _step3.value;

                    this.renderHorizontalConnection(leftNode, rightNode, skipConnectorStroke, elements);
                    leftNode = rightNode;
                }
            } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return) {
                        _iterator3.return();
                    }
                } finally {
                    if (_didIteratorError3) {
                        throw _iteratorError3;
                    }
                }
            }

            this.renderHorizontalConnection(leftNode, destinationNodes[0], skipConnectorStroke, elements);

            //--------------------------------------------------------------------------
            //  Work out the extents of source and dest space

            var rightmostSource = sourceNodes[0].x;
            var leftmostDestination = destinationNodes[0].x;

            for (var i = 1; i < sourceNodes.length; i++) {
                rightmostSource = Math.max(rightmostSource, sourceNodes[i].x);
            }

            for (var _i2 = 1; _i2 < destinationNodes.length; _i2++) {
                leftmostDestination = Math.min(leftmostDestination, destinationNodes[_i2].x);
            }

            //--------------------------------------------------------------------------
            //  "Collapse" from the source node(s) down toward the first skipped

            leftNode = sourceNodes[0];
            rightNode = skippedNodes[0];

            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
                for (var _iterator4 = sourceNodes.slice(1)[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                    leftNode = _step4.value;

                    var midPointX = Math.round((_PipelineGraphModel.MATRIOSKA_PATHS ? leftNode.x : rightmostSource) + halfSpacingH);
                    var _leftNodeRadius = leftNode.isPlaceholder ? terminalRadius : nodeRadius;
                    var _key = connectorKey(leftNode, rightNode);

                    var x1 = leftNode.x + _leftNodeRadius - _SvgSpinner.strokeWidth / 2;
                    var y1 = leftNode.y;
                    var x2 = midPointX;
                    var y2 = rightNode.y;

                    var _pathData = 'M ' + x1 + ' ' + y1 + this.svgCurve(x1, y1, x2, y2, midPointX, curveRadius);

                    elements.push(_react2.default.createElement('path', _extends({}, connectorStroke, { key: _key, d: _pathData, fill: 'none' })));
                }

                //--------------------------------------------------------------------------
                //  "Expand" from the last skipped node toward the destination nodes
            } catch (err) {
                _didIteratorError4 = true;
                _iteratorError4 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                        _iterator4.return();
                    }
                } finally {
                    if (_didIteratorError4) {
                        throw _iteratorError4;
                    }
                }
            }

            leftNode = lastSkippedNode;
            rightNode = destinationNodes[0];

            var _iteratorNormalCompletion5 = true;
            var _didIteratorError5 = false;
            var _iteratorError5 = undefined;

            try {
                for (var _iterator5 = destinationNodes.slice(1)[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
                    rightNode = _step5.value;

                    var _midPointX2 = Math.round((_PipelineGraphModel.MATRIOSKA_PATHS ? rightNode.x : leftmostDestination) - halfSpacingH);
                    var _rightNodeRadius = rightNode.isPlaceholder ? terminalRadius : nodeRadius;
                    var _key2 = connectorKey(leftNode, rightNode);

                    var _x2 = _midPointX2;
                    var _y = leftNode.y;
                    var _x3 = rightNode.x - _rightNodeRadius + _SvgSpinner.strokeWidth / 2;
                    var _y2 = rightNode.y;

                    var _pathData2 = 'M ' + _x2 + ' ' + _y + this.svgCurve(_x2, _y, _x3, _y2, _midPointX2, curveRadius);

                    elements.push(_react2.default.createElement('path', _extends({}, connectorStroke, { key: _key2, d: _pathData2, fill: 'none' })));
                }

                //--------------------------------------------------------------------------
                //  "Main" curve from top of source nodes, around skipped nodes, to top of dest nodes
            } catch (err) {
                _didIteratorError5 = true;
                _iteratorError5 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion5 && _iterator5.return) {
                        _iterator5.return();
                    }
                } finally {
                    if (_didIteratorError5) {
                        throw _iteratorError5;
                    }
                }
            }

            leftNode = sourceNodes[0];
            rightNode = destinationNodes[0];

            var leftNodeRadius = leftNode.isPlaceholder ? terminalRadius : nodeRadius;
            var rightNodeRadius = rightNode.isPlaceholder ? terminalRadius : nodeRadius;
            var key = connectorKey(leftNode, rightNode);

            var skipHeight = nodeSpacingV * 0.5;
            var controlOffsetUpper = curveRadius * 1.54;
            var controlOffsetLower = skipHeight * 0.257;
            var controlOffsetMid = skipHeight * 0.2;
            var inflectiontOffset = Math.round(skipHeight * 0.7071); // cos(45º)-ish

            // Start point
            var p1x = leftNode.x + leftNodeRadius - _SvgSpinner.strokeWidth / 2;
            var p1y = leftNode.y;

            // Begin curve down point
            var p2x = Math.round(skippedNodes[0].x - halfSpacingH);
            var p2y = p1y;
            var c1x = p2x + controlOffsetUpper;
            var c1y = p2y;

            // End curve down point
            var p4x = skippedNodes[0].x;
            var p4y = p1y + skipHeight;
            var c4x = p4x - controlOffsetLower;
            var c4y = p4y;

            // Curve down midpoint / inflection
            var p3x = skippedNodes[0].x - inflectiontOffset;
            var p3y = skippedNodes[0].y + inflectiontOffset;
            var c2x = p3x - controlOffsetMid;
            var c2y = p3y - controlOffsetMid;
            var c3x = p3x + controlOffsetMid;
            var c3y = p3y + controlOffsetMid;

            // Begin curve up point
            var p5x = lastSkippedNode.x;
            var p5y = p4y;
            var c5x = p5x + controlOffsetLower;
            var c5y = p5y;

            // End curve up point
            var p7x = Math.round(lastSkippedNode.x + halfSpacingH);
            var p7y = rightNode.y;
            var c8x = p7x - controlOffsetUpper;
            var c8y = p7y;

            // Curve up midpoint / inflection
            var p6x = lastSkippedNode.x + inflectiontOffset;
            var p6y = lastSkippedNode.y + inflectiontOffset;
            var c6x = p6x - controlOffsetMid;
            var c6y = p6y + controlOffsetMid;
            var c7x = p6x + controlOffsetMid;
            var c7y = p6y - controlOffsetMid;

            // End point
            var p8x = rightNode.x - rightNodeRadius + _SvgSpinner.strokeWidth / 2;
            var p8y = rightNode.y;

            var pathData = 'M ' + p1x + ' ' + p1y + ('L ' + p2x + ' ' + p2y) + ( // 1st horizontal
            'C ' + c1x + ' ' + c1y + ' ' + c2x + ' ' + c2y + ' ' + p3x + ' ' + p3y) + ( // Curve down (upper)
            'C ' + c3x + ' ' + c3y + ' ' + c4x + ' ' + c4y + ' ' + p4x + ' ' + p4y) + ( // Curve down (lower)
            'L ' + p5x + ' ' + p5y) + ( // 2nd horizontal
            'C ' + c5x + ' ' + c5y + ' ' + c6x + ' ' + c6y + ' ' + p6x + ' ' + p6y) + ( // Curve up (lower)
            'C ' + c7x + ' ' + c7y + ' ' + c8x + ' ' + c8y + ' ' + p7x + ' ' + p7y) + ( // Curve up (upper)
            'L ' + p8x + ' ' + p8y) + // Last horizontal
            '';

            elements.push(_react2.default.createElement('path', _extends({}, connectorStroke, { key: key, d: pathData, fill: 'none' })));
        }

        /**
         * Simple straight connection.
         *
         * Adds all the SVG components to the elements list.
         */

    }, {
        key: 'renderHorizontalConnection',
        value: function renderHorizontalConnection(leftNode, rightNode, connectorStroke, elements) {
            var _state$layout5 = this.state.layout,
                nodeRadius = _state$layout5.nodeRadius,
                terminalRadius = _state$layout5.terminalRadius;

            var leftNodeRadius = leftNode.isPlaceholder ? terminalRadius : nodeRadius;
            var rightNodeRadius = rightNode.isPlaceholder ? terminalRadius : nodeRadius;

            var key = connectorKey(leftNode, rightNode);

            var x1 = leftNode.x + leftNodeRadius - _SvgSpinner.strokeWidth / 2;
            var x2 = rightNode.x - rightNodeRadius + _SvgSpinner.strokeWidth / 2;
            var y = leftNode.y;

            elements.push(_react2.default.createElement('line', _extends({}, connectorStroke, { key: key, x1: x1, y1: y, x2: x2, y2: y })));
        }

        /**
         * A direct curve between two nodes in adjacent columns.
         *
         * Adds all the SVG components to the elements list.
         */

    }, {
        key: 'renderBasicCurvedConnection',
        value: function renderBasicCurvedConnection(leftNode, rightNode, midPointX, elements) {
            var _state$layout6 = this.state.layout,
                nodeRadius = _state$layout6.nodeRadius,
                terminalRadius = _state$layout6.terminalRadius,
                curveRadius = _state$layout6.curveRadius,
                connectorStrokeWidth = _state$layout6.connectorStrokeWidth;

            var leftNodeRadius = leftNode.isPlaceholder ? terminalRadius : nodeRadius;
            var rightNodeRadius = rightNode.isPlaceholder ? terminalRadius : nodeRadius;

            var key = connectorKey(leftNode, rightNode);

            var leftPos = {
                x: leftNode.x + leftNodeRadius - _SvgSpinner.strokeWidth / 2,
                y: leftNode.y
            };

            var rightPos = {
                x: rightNode.x - rightNodeRadius + _SvgSpinner.strokeWidth / 2,
                y: rightNode.y
            };

            // Stroke props common to straight / curved connections
            var connectorStroke = {
                className: 'pipeline-connector',
                strokeWidth: connectorStrokeWidth
            };

            var pathData = 'M ' + leftPos.x + ' ' + leftPos.y + this.svgCurve(leftPos.x, leftPos.y, rightPos.x, rightPos.y, midPointX, curveRadius);

            elements.push(_react2.default.createElement('path', _extends({}, connectorStroke, { key: key, d: pathData, fill: 'none' })));
        }

        /**
         * Generates an SVG path string for the "vertical" S curve used to connect nodes in adjacent columns.
         */

    }, {
        key: 'svgCurve',
        value: function svgCurve(x1, y1, x2, y2, midPointX, curveRadius) {
            var verticalDirection = Math.sign(y2 - y1); // 1 == curve down, -1 == curve up
            var w1 = midPointX - curveRadius - x1 + curveRadius * verticalDirection;
            var w2 = x2 - curveRadius - midPointX - curveRadius * verticalDirection;
            var v = y2 - y1 - 2 * curveRadius * verticalDirection; // Will be -ive if curve up
            var cv = verticalDirection * curveRadius;

            return ' l ' + w1 + ' 0' + ( // first horizontal line
            ' c ' + curveRadius + ' 0 ' + curveRadius + ' ' + cv + ' ' + curveRadius + ' ' + cv) + ( // turn
            ' l 0 ' + v) + ( // vertical line
            ' c 0 ' + cv + ' ' + curveRadius + ' ' + cv + ' ' + curveRadius + ' ' + cv) + ( // turn again
            ' l ' + w2 + ' 0') // second horizontal line
            ;
        }

        /**
         * Generate the SVG elements to represent a node.
         *
         * Adds all the SVG components to the elements list.
         */

    }, {
        key: 'renderNode',
        value: function renderNode(node, elements) {
            var _this3 = this;

            var nodeIsSelected = false;
            var _state$layout7 = this.state.layout,
                nodeRadius = _state$layout7.nodeRadius,
                connectorStrokeWidth = _state$layout7.connectorStrokeWidth,
                terminalRadius = _state$layout7.terminalRadius;

            var key = node.key;

            var groupChildren = [];

            if (node.isPlaceholder === true) {
                groupChildren.push(_react2.default.createElement('circle', { r: terminalRadius, className: 'pipeline-node-terminal' }));
            } else {
                var _node$stage = node.stage,
                    _node$stage$completeP = _node$stage.completePercent,
                    completePercent = _node$stage$completeP === undefined ? 0 : _node$stage$completeP,
                    title = _node$stage.title,
                    state = _node$stage.state;

                var resultClean = (0, _StatusIndicator.decodeResultValue)(state);

                groupChildren.push((0, _StatusIndicator.getGroupForResult)(resultClean, completePercent, nodeRadius));

                if (title) {
                    groupChildren.push(_react2.default.createElement(
                        'title',
                        null,
                        title
                    ));
                }

                nodeIsSelected = this.stageIsSelected(node.stage);
            }

            // Set click listener and link cursor only for nodes we want to be clickable
            var clickableProps = {};

            if (node.isPlaceholder === false && node.stage.state !== 'skipped') {
                clickableProps.cursor = 'pointer';
                clickableProps.onClick = function () {
                    return _this3.nodeClicked(node);
                };
            }

            // Add an invisible click/touch/mouseover target, coz the nodes are small and (more importantly)
            // many are hollow.
            groupChildren.push(_react2.default.createElement('circle', _extends({ r: nodeRadius + 2 * connectorStrokeWidth, className: 'pipeline-node-hittarget', fillOpacity: '0', stroke: 'none' }, clickableProps)));

            // Most of the nodes are in shared code, so they're rendered at 0,0. We transform with a <g> to position them
            var groupProps = {
                key: key,
                transform: 'translate(' + node.x + ',' + node.y + ')',
                className: nodeIsSelected ? 'pipeline-node-selected' : 'pipeline-node'
            };

            elements.push(_react2.default.createElement.apply(_react2.default, ['g', groupProps].concat(groupChildren)));
        }

        /**
         * Generates SVG for visual highlight to show which node is selected.
         *
         * Adds all the SVG components to the elements list.
         */

    }, {
        key: 'renderSelectionHighlight',
        value: function renderSelectionHighlight(elements) {
            var _state$layout8 = this.state.layout,
                nodeRadius = _state$layout8.nodeRadius,
                connectorStrokeWidth = _state$layout8.connectorStrokeWidth;

            var highlightRadius = Math.ceil(nodeRadius + 0.5 * connectorStrokeWidth + 1);
            var selectedNode = null;

            var _iteratorNormalCompletion6 = true;
            var _didIteratorError6 = false;
            var _iteratorError6 = undefined;

            try {
                columnLoop: for (var _iterator6 = this.state.nodeColumns[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
                    var column = _step6.value;
                    var _iteratorNormalCompletion7 = true;
                    var _didIteratorError7 = false;
                    var _iteratorError7 = undefined;

                    try {
                        for (var _iterator7 = column.rows[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
                            var row = _step7.value;
                            var _iteratorNormalCompletion8 = true;
                            var _didIteratorError8 = false;
                            var _iteratorError8 = undefined;

                            try {
                                for (var _iterator8 = row[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
                                    var node = _step8.value;

                                    if (node.isPlaceholder === false && this.stageIsSelected(node.stage)) {
                                        selectedNode = node;
                                        break columnLoop;
                                    }
                                }
                            } catch (err) {
                                _didIteratorError8 = true;
                                _iteratorError8 = err;
                            } finally {
                                try {
                                    if (!_iteratorNormalCompletion8 && _iterator8.return) {
                                        _iterator8.return();
                                    }
                                } finally {
                                    if (_didIteratorError8) {
                                        throw _iteratorError8;
                                    }
                                }
                            }
                        }
                    } catch (err) {
                        _didIteratorError7 = true;
                        _iteratorError7 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion7 && _iterator7.return) {
                                _iterator7.return();
                            }
                        } finally {
                            if (_didIteratorError7) {
                                throw _iteratorError7;
                            }
                        }
                    }
                }
            } catch (err) {
                _didIteratorError6 = true;
                _iteratorError6 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion6 && _iterator6.return) {
                        _iterator6.return();
                    }
                } finally {
                    if (_didIteratorError6) {
                        throw _iteratorError6;
                    }
                }
            }

            if (selectedNode) {
                var transform = 'translate(' + selectedNode.x + ' ' + selectedNode.y + ')';

                elements.push(_react2.default.createElement(
                    'g',
                    { className: 'pipeline-selection-highlight', transform: transform, key: 'selection-highlight' },
                    _react2.default.createElement('circle', { className: 'white-highlight', r: highlightRadius - 2, strokeWidth: 10 }),
                    _react2.default.createElement('circle', { r: highlightRadius, strokeWidth: 2 })
                ));
            }
        }

        /**
         * Is this stage currently selected?
         */

    }, {
        key: 'stageIsSelected',
        value: function stageIsSelected(stage) {
            var selectedStage = this.state.selectedStage;

            return selectedStage && stage && selectedStage.id === stage.id;
        }

        /**
         * Is this any child of this stage currently selected?
         */

    }, {
        key: 'stageChildIsSelected',
        value: function stageChildIsSelected(stage) {
            if (stage) {
                var children = stage.children;
                var _selectedStage = this.state.selectedStage;


                if (children && _selectedStage) {
                    var _iteratorNormalCompletion9 = true;
                    var _didIteratorError9 = false;
                    var _iteratorError9 = undefined;

                    try {
                        for (var _iterator9 = children[Symbol.iterator](), _step9; !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
                            var childStage = _step9.value;

                            var currentStage = childStage;

                            while (currentStage) {
                                if (currentStage.id === _selectedStage.id) {
                                    return true;
                                }
                                currentStage = currentStage.nextSibling;
                            }
                        }
                    } catch (err) {
                        _didIteratorError9 = true;
                        _iteratorError9 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion9 && _iterator9.return) {
                                _iterator9.return();
                            }
                        } finally {
                            if (_didIteratorError9) {
                                throw _iteratorError9;
                            }
                        }
                    }
                }
            }

            return false;
        }
    }, {
        key: 'nodeClicked',
        value: function nodeClicked(node) {
            if (node.isPlaceholder === false && node.stage.state !== 'skipped') {
                var stage = node.stage;
                var listener = this.props.onNodeClick;

                if (listener) {
                    listener(stage.name, stage.id);
                }

                // Update selection
                this.setState({ selectedStage: stage });
            }
        }
    }, {
        key: 'render',
        value: function render() {
            var _this4 = this;

            var _state = this.state,
                _state$nodeColumns = _state.nodeColumns,
                nodeColumns = _state$nodeColumns === undefined ? [] : _state$nodeColumns,
                _state$connections = _state.connections,
                connections = _state$connections === undefined ? [] : _state$connections,
                _state$bigLabels = _state.bigLabels,
                bigLabels = _state$bigLabels === undefined ? [] : _state$bigLabels,
                _state$smallLabels = _state.smallLabels,
                smallLabels = _state$smallLabels === undefined ? [] : _state$smallLabels,
                measuredWidth = _state.measuredWidth,
                measuredHeight = _state.measuredHeight;

            // Without these we get fire, so they're hardcoded

            var outerDivStyle = {
                position: 'relative', // So we can put the labels where we need them
                overflow: 'visible' // So long labels can escape this component in layout
            };

            var visualElements = []; // Buffer for children of the SVG

            connections.forEach(function (connection) {
                _this4.renderCompositeConnection(connection, visualElements);
            });

            this.renderSelectionHighlight(visualElements);

            var _iteratorNormalCompletion10 = true;
            var _didIteratorError10 = false;
            var _iteratorError10 = undefined;

            try {
                for (var _iterator10 = nodeColumns[Symbol.iterator](), _step10; !(_iteratorNormalCompletion10 = (_step10 = _iterator10.next()).done); _iteratorNormalCompletion10 = true) {
                    var column = _step10.value;
                    var _iteratorNormalCompletion11 = true;
                    var _didIteratorError11 = false;
                    var _iteratorError11 = undefined;

                    try {
                        for (var _iterator11 = column.rows[Symbol.iterator](), _step11; !(_iteratorNormalCompletion11 = (_step11 = _iterator11.next()).done); _iteratorNormalCompletion11 = true) {
                            var row = _step11.value;
                            var _iteratorNormalCompletion12 = true;
                            var _didIteratorError12 = false;
                            var _iteratorError12 = undefined;

                            try {
                                for (var _iterator12 = row[Symbol.iterator](), _step12; !(_iteratorNormalCompletion12 = (_step12 = _iterator12.next()).done); _iteratorNormalCompletion12 = true) {
                                    var node = _step12.value;

                                    this.renderNode(node, visualElements);
                                }
                            } catch (err) {
                                _didIteratorError12 = true;
                                _iteratorError12 = err;
                            } finally {
                                try {
                                    if (!_iteratorNormalCompletion12 && _iterator12.return) {
                                        _iterator12.return();
                                    }
                                } finally {
                                    if (_didIteratorError12) {
                                        throw _iteratorError12;
                                    }
                                }
                            }
                        }
                    } catch (err) {
                        _didIteratorError11 = true;
                        _iteratorError11 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion11 && _iterator11.return) {
                                _iterator11.return();
                            }
                        } finally {
                            if (_didIteratorError11) {
                                throw _iteratorError11;
                            }
                        }
                    }
                }
            } catch (err) {
                _didIteratorError10 = true;
                _iteratorError10 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion10 && _iterator10.return) {
                        _iterator10.return();
                    }
                } finally {
                    if (_didIteratorError10) {
                        throw _iteratorError10;
                    }
                }
            }

            return _react2.default.createElement(
                'div',
                { style: outerDivStyle, className: 'PipelineGraph' },
                _react2.default.createElement(
                    'svg',
                    { width: measuredWidth, height: measuredHeight },
                    visualElements
                ),
                bigLabels.map(function (label) {
                    return _this4.renderBigLabel(label);
                }),
                smallLabels.map(function (label) {
                    return _this4.renderSmallLabel(label);
                })
            );
        }
    }]);

    return PipelineGraph;
}(_react.Component);

PipelineGraph.propTypes = {
    stages: _react.PropTypes.array,
    layout: _react.PropTypes.object,
    onNodeClick: _react.PropTypes.func
};
//# sourceMappingURL=PipelineGraph.js.map
